/*
 * misc_runner
 * $Revision: 1.1 $
 *
 * Copyright (c) 2003 Timo Savola <timo.savola@movial.fi>
 *
 * misc_runner is licensed under GPL.
 * You can read the full license at http://www.gnu.org/licenses/gpl.txt
 */

#define _GNU_SOURCE

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <errno.h>
#include <string.h>
#include <ctype.h>
#include <limits.h>
#include <time.h>
#include <sys/time.h>

#define SBRSH_CMD   "sbrsh"
#define CONFIG_NAME "/targets/links/scratchbox.config"
#define LOGNAME_FMT "/tmp/cputransp_%s.log"

#define ERROR_CODE  126

static const char *progname;

static int read_line(FILE *file,
		     char *buf,
		     int size)
{
	int comm = 0;
	int c, i = 0;

	if (feof(file)) {
		return -1;
	}

	while (1) {
		c = fgetc(file);
		if (c == EOF || c == '\n') {
			break;
		}
		if (c == '#') {
			comm = 1;
		}
		if (!comm && i < size - 1) {
			*buf++ = c;
			i++;
		}
	}
	*buf = '\0';

	return i;
}

static int is_set(char *var)
{
	char *str;

	str = getenv(var);

	return (str && strlen(str) > 0 && *str != '0');
}

static void read_config(void)
{
	FILE *file;
	char str[1024];

	file = fopen(CONFIG_NAME, "r");
	if (!file) {
		fprintf(stderr, "%s: cannot open file " CONFIG_NAME " (%s)\n",
			progname, strerror(errno));
		return;
	}

	while (read_line(file, str, sizeof (str)) >= 0) {
		if (strchr(str, '=') != NULL) {
			putenv(strdup(str));
		}
	}

	fclose(file);
}

static char *build_log_name(void)
{
	char *user, *str;
	int len;

	user = getenv("USER");
	if (!user) {
		user = "";
	}

	len = strlen(LOGNAME_FMT) - 2 + strlen(user) + 1;

	str = malloc(len);
	if (!str) {
		fprintf(stderr, "%s: %s\n", progname, strerror(errno));
		exit(ERROR_CODE);
	}

	snprintf(str, len, LOGNAME_FMT, user);

	return str;
}

static void print_log(const char *logname,
		      const char *pwd,
		      char **args)
{
	FILE *file;
	struct timeval tv;
	struct tm *t;
	int i;

	file = fopen(logname, "a");
	if (!file) {
		fprintf(stderr, "%s: cannot append to file %s (%s)\n",
			progname, logname, strerror(errno));
		return;
	}

	gettimeofday(&tv, NULL);
	t = gmtime(&tv.tv_sec);

	fprintf(file, "[%02d.%02d.%04d %02d:%02d:%02d] ",
		t->tm_mday, t->tm_mon + 1, 1900 + t->tm_year,
		t->tm_hour, t->tm_min, t->tm_sec);

	fprintf(file, "pwd: '%s' ", pwd);

	fprintf(file, "cmd: '");
	for (i = 0; args[i]; i++) {
		fprintf(file, i ? " %s" : "%s", args[i]);
	}
	fprintf(file, "'\n");

	fclose(file);
}

static void fix_args(char **args)
{
	FILE *file;
	char buf[PATH_MAX];
	int i;

	if (!args[0] || !args[1] || strcmp(args[0], args[1]) != 0) {
		return;
	}

	file = fopen(args[0], "r");
	if (!file) {
		fprintf(stderr, "%s: cannot open file %s (%s)\n",
			progname, args[0], strerror(errno));
		exit(ERROR_CODE);
	}

	if (fread(buf, 1, 2, file) != 2) {
		fprintf(stderr, "%s: cannot read file %s (%s)\n",
			progname, args[0], strerror(errno));
		exit(ERROR_CODE);
	}

	if (strncmp(buf, "#!", 2) == 0) {
		if (read_line(file, buf, sizeof (buf)) > 0) {
			for (i = 0; isspace(buf[i]); i++);

			if (buf[i]) {
				args[0] = strdup(&buf[i]);
			}
		}
	}

	fclose(file);
}

static char **build_args(char *pwd,
			 int argc,
			 char **argv)
{
	char *target, **vec;
	int i;

	target = getenv("SBOX_TARGET_NAME");
	if (!target || strlen(target) == 0) {
		fprintf(stderr, "%s: SBOX_TARGET_NAME not defined\n",
			progname);
		exit(ERROR_CODE);
	}

	vec = calloc(4 + (argc - 1) + 1, sizeof (char *));
	if (!vec) {
		fprintf(stderr, "%s: %s\n", progname, strerror(errno));
		exit(ERROR_CODE);
	}

	vec[0] = SBRSH_CMD;
	vec[1] = target;
	vec[2] = "-d";
	vec[3] = pwd;

	for (i = 1; i < argc; i++) {
		vec[4 + (i - 1)] = argv[i];
	}

	fix_args(&vec[4]);

	return vec;
}

int main(int argc,
	 char **argv)
{
	char *logname, *pwd, **argvec;

	progname = argv[0];

	if (argc < 2) {
		fprintf(stderr, "%s: insufficient arguments\n", progname);
		exit(ERROR_CODE);
	}

	if (is_set("SBOX_DISABLE_CPUTRANSPARENCY")) {
		fprintf(stderr, "%s: cannot execute binary file\n", argv[1]);
		exit(ERROR_CODE);
	}

	read_config();

	logname = build_log_name();

	if (is_set("SBOX_CPUTRANSPARENCY_VERBOSE")) {
		fprintf(stderr, "\n ### INFO ###  cpuTransparency!"
			"  See %s for details!\n", logname);
	}

	pwd = get_current_dir_name();
	if (!pwd) {
		fprintf(stderr, "%s: %s\n", progname, strerror(errno));
		exit(ERROR_CODE);
	}

	argvec = build_args(pwd, argc, argv);

	print_log(logname, pwd, &argvec[4]);

	execvp(argvec[0], argvec);

	fprintf(stderr, "%s: cannot execute " SBRSH_CMD " (%s)\n",
		progname, strerror(errno));
	return ERROR_CODE;
}
